# -*- coding: utf-8 -*-
"""
Contiene clases que representan figuras geométricas

@author: Michael
@date:   may 2019
"""

from abc import ABC, abstractmethod


class Figura(ABC):
    """
    Clase que representa una figura abstracta

    Attributes:
        _tipo: string que define el tipo de figura
        _pos: la posición donde se dibujó la figura,
              o None si nunca fue dibujada
    """

    def __init__(self):
        """ Crea la figura"""
        self._tipo: str = "Error"
        self._pos: str | None = None

    def __str__(self) -> str:
        """ Retorna el tipo de figura"""
        return self._tipo

    @abstractmethod
    def dibuja(self, pos: str):
        """ Dibuja la figura"""

    @abstractmethod
    def borra(self):
        """ Borra la figura"""

    def get_posicion(self) -> str | None:
        """ Retorna la posición donde se dibujó la figura,
            o None si nunca fue dibujada"""
        return self._pos


class Circulo(Figura):
    """ Representa un círculo"""

    def __init__(self):
        """ Crea el círculo"""
        super().__init__()
        self._tipo = "Círculo"

    def dibuja(self, pos: str):
        """ Dibuja la figura"""
        self._pos = pos
        print(f"Dibuja un círculo en {pos}")

    def borra(self):
        """ Borra la figura"""
        print("Borra un círculo")


class Cuadrado(Figura):
    """ Representa un cuadrado"""

    def __init__(self):
        """ Crea el cuadrado"""
        super().__init__()
        self._tipo = "Cuadrado"

    def dibuja(self, pos: str):
        """ Dibuja el cudrado"""
        self._pos = pos
        print(f"Dibuja un cuadrado en {pos}")

    def borra(self):
        """ Borra la figura"""
        print("Borra un cuadrado")


class Poligono(Figura):
    """ Representa un polígono"""

    def __init__(self):
        """ Crea el polígono"""
        super().__init__()
        self._tipo = "Polígono"

    def dibuja(self, pos: str):
        """ Dibuja la figura"""
        self._pos = pos
        print(f"Dibuja un polígono en {pos}")

    def borra(self):
        """ Borra la figura"""
        print("Borra un polígono")


def mueve_fig(fig: Figura, pos: str):
    """ Mueve la figura a la posición pos"""
    fig.borra()
    fig.dibuja(pos)


def reemplaza_fig(fig1: Figura, fig2: Figura):
    """ Reemplaza fig1 por fig2"""
    pos: str | None = fig1.get_posicion()
    # Reemplaza la figura si estaba ya dibujada
    if pos is not None:
        fig1.borra()
        fig2.dibuja(pos)


def muestra_lista(lista: list[Figura]):
    """
    Muestra en pantalla una lista de figuras
    """
    for fig in lista:
        print(fig)


def main():
    """
    Prueba las figuras
    """
    cir = Circulo()
    pol = Poligono()

    # mueve_fig(cir, pos)
    # mueve_fig(pol, pos)
    reemplaza_fig(cir, pol)

    lista = [
        Circulo(),
        Cuadrado(),
        Cuadrado(),
        Poligono(),
        Circulo(),
        Poligono(),
        Cuadrado(),
        ]
    muestra_lista(lista)
