# -*- coding: utf-8 -*-
"""
Contiene la clase Vehiculo y varias subclases

@author: Michael
@date: abr 2019
"""

from abc import ABC, abstractmethod


class Vehiculo(ABC):
    """
    Clase abstracta que representa un vehículo cualquiera

    Class Attributes:
        ROJO: un color
        VERDE: un color
        AZUL: un color
        __ultimo_num_serie: el último número de serie asignado a un vehículo

    Instance Attributes:
        _color: el color del que se pinta un vehículo (ROJO, VERDE o AZUL)
        __num_serie: número de serie del vehículo
    """
    # colores de los que se puede pintar un vehículo
    ROJO: int = 1
    VERDE: int = 2
    AZUL: int = 3

    __ultimo_num_serie: int = 0

    def __init__(self, color: int):
        """
        Construye un vehiculo dándole un número de serie único
        Args:
            color: el color del vehículo
        """
        self._color = color
        # obtener un nuevo número de serie
        Vehiculo.__ultimo_num_serie += 1
        self.__num_serie = Vehiculo.__ultimo_num_serie

    def get_color(self) -> int:
        """
        Retorna el color del vehículo
        """
        return self._color

    def get_num_serie(self) -> int:
        """
        Retorna el número de serie del vehículo
        """
        return self.__num_serie

    def pinta(self, nuevo_color: int):
        """
        Pinta el vehículo de un color
        Args:
            nuevo_color: color con el que pintar el vehículo
        """
        self._color = nuevo_color

    def __str__(self) -> str:
        """
        Retorna un string que describe el vehículo
        """
        nombre_color: list[str] = ["ROJO", "VERDE", "AZUL"]
        return f"Vehiculo {self.__num_serie} color=" + \
            f"{nombre_color[self._color-1]}"

    @abstractmethod
    def medio_por_el_que_se_mueve(self) -> str:
        """
        Retorna el medio por el que se mueve el vehiculo
        """
        pass


class Coche(Vehiculo):
    """
    Clase que representa un coche

    Instance Attributes:
        _cilindrada: cilindrada del coche

    Inherited class Attributes:
        ROJO: un color
        VERDE: un color
        AZUL: un color
        __ultimo_num_serie: el último número de serie asignado a un vehículo

    Inherited instance Attributes:
        _color: el color del que se pinta un vehículo (ROJO, VERDE o AZUL)
        __num_serie: número de serie del vehículo
                     (no lo puede usar por comenzar con __)

    """

    def __init__(self, color: int, cilindrada: int):
        """
        Crea el coche con el color y la cilindrada indicados
        Args:
            color: el color del coche
            cilindrada: la cilindrada del coche en cc
        """
        super().__init__(color)
        self._cilindrada = cilindrada

    def get_cilindrada(self) -> int:
        """
        Retorna la cilindrada del coche
        """
        return self._cilindrada

    def cambia_cilindrada(self, cilindrada: int):
        """
        Cambia la cilindrada del coche al valor indicado, en cc
        """
        self._cilindrada = cilindrada

    def __str__(self) -> str:
        """
        Retorna un string que describe el coche, incluyendo su cilindrada
        """
        return super().__str__()+f", cil={self._cilindrada}"

    def medio_por_el_que_se_mueve(self) -> str:
        """
        Retorna el medio por el que se mueve el vehiculo
        """
        return "Terrestre"


class Barco(Vehiculo):
    """
    Clase que representa un barco

    Instance Attributes:

    Inherited class Attributes:
        ROJO: un color
        VERDE: un color
        AZUL: un color
        __ultimo_num_serie: el último número de serie asignado a un vehículo

    Inherited instance Attributes:
        _color: el color del que se pinta un vehículo (ROJO, VERDE o AZUL)
        __num_serie: número de serie del vehículo (no lo puede usar)

    """

#    def __init__(self, color: int):
#        """
#        Crea el barco con el color indicado
#        Args:
#            color: el color del coche
#        """
#        super().__init__(color)

    def medio_por_el_que_se_mueve(self) -> str:
        """
        Retorna el medio por el que se mueve el vehiculo
        """
        return "Acuático"


def main():
    """
    Crea unos vehículos y muestra sus datos en pantalla
    """
    # v_1 = Vehiculo(Vehiculo.ROJO) fallaría
    c_1 = Coche(Vehiculo.VERDE, 2000)
    b_1 = Barco(Vehiculo.AZUL)
    print(c_1)
    print(b_1)
    c_1.pinta(Vehiculo.AZUL)
    c_1.cambia_cilindrada(2500)
    print(c_1)
    print(c_1.medio_por_el_que_se_mueve())


if __name__ == "__main__":
    main()
